'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _utils = require('./utils');

var _pathToRegexp = require('path-to-regexp');

var _pathToRegexp2 = _interopRequireDefault(_pathToRegexp);

var _queryString = require('query-string');

var _queryString2 = _interopRequireDefault(_queryString);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultPathPattern = /.*/;
var defaultMatch = function defaultMatch() {
  return false;
};
var paramDefaults = {
  parse: function parse(value) {
    return value;
  },
  stringify: function stringify(value) {
    return String(value);
  },
  notSetValue: null
};

var isParamOptional = function isParamOptional(param, pathParams) {
  var pathParam = pathParams.find(function (pathParam) {
    return pathParam.name === param.name;
  });
  return pathParam ? pathParam.optional : true;
};

var createParamConditions = function createParamConditions(params) {
  var paramNames = params.map(function (param) {
    return param.name;
  });

  return {
    isPathParam: function isPathParam(name) {
      return paramNames.includes(name);
    },
    isQueryParam: function isQueryParam(name) {
      return !paramNames.includes(name);
    }
  };
};

var createPathPattern = function createPathPattern(path, pathParams) {
  return path !== undefined ? (0, _pathToRegexp2.default)(path, pathParams) : defaultPathPattern;
};

var createParams = function createParams() {
  var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var pathParams = arguments[1];
  return (0, _entries2.default)(params).map(function (_ref) {
    var _ref2 = (0, _slicedToArray3.default)(_ref, 2),
        key = _ref2[0],
        param = _ref2[1];

    return (0, _extends3.default)({ key: key }, paramDefaults, param);
  }).map(function (param) {
    return [param.key, {
      key: param.key,
      name: param.name,
      action: param.action,
      selector: param.selector,
      parse: param.parse,
      stringify: param.stringify,
      notSetValue: param.notSetValue,
      optional: isParamOptional(param, pathParams)
    }];
  }).reduce(_utils.createObject, {});
};

var createMatch = function createMatch() {
  var match = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultMatch;
  var params = arguments[1];
  return function (state) {
    var baseMatch = match(state);
    var requiredMatch = (0, _values2.default)(params).filter(function (param) {
      return !param.optional;
    }).every(function (param) {
      return param.selector(state) !== param.notSetValue;
    });

    return baseMatch && requiredMatch;
  };
};

var createGetLocation = function createGetLocation(path, pathParams) {
  if (path !== undefined) {
    var _createParamCondition = createParamConditions(pathParams),
        isPathParam = _createParamCondition.isPathParam,
        isQueryParam = _createParamCondition.isQueryParam;

    var getPath = _pathToRegexp2.default.compile(path);
    var getSearch = _queryString2.default.stringify;

    return function (values, location) {
      return (0, _utils.normalizeLocation)((0, _extends3.default)({}, location, {
        pathname: getPath((0, _utils.keyFilter)(values, isPathParam)),
        search: getSearch((0, _utils.keyFilter)(values, isQueryParam))
      }));
    };
  } else {
    return function (values, location) {
      return location;
    };
  }
};

var createGetParamValues = function createGetParamValues(pathPattern, pathParams) {
  var _createParamCondition2 = createParamConditions(pathParams),
      isPathParam = _createParamCondition2.isPathParam,
      isQueryParam = _createParamCondition2.isQueryParam;

  var getPathValues = function getPathValues(location) {
    var result = pathPattern.exec(location.pathname) || [];
    var values = result.splice(1);

    return (0, _utils.zip)(pathParams, values).filter(function (_ref3) {
      var _ref4 = (0, _slicedToArray3.default)(_ref3, 2),
          value = _ref4[1];

      return value !== undefined;
    }).map(function (_ref5) {
      var _ref6 = (0, _slicedToArray3.default)(_ref5, 2),
          param = _ref6[0],
          value = _ref6[1];

      return [param.name, decodeURIComponent(value)];
    }).reduce(_utils.createObject, {});
  };
  var getSearchValues = function getSearchValues(location) {
    return _queryString2.default.parse(location.search);
  };

  return function (location) {
    return (0, _extends3.default)({}, (0, _utils.keyFilter)(getPathValues(location), isPathParam), (0, _utils.keyFilter)(getSearchValues(location), isQueryParam));
  };
};

var createLocationMatch = function createLocationMatch(pathPattern) {
  return function (location) {
    return pathPattern.test(location.pathname);
  };
};

exports.default = function (routes) {
  return routes.map(function (route) {
    var pathParams = [];
    var pathPattern = createPathPattern(route.path, pathParams);
    var params = createParams(route.params, pathParams);

    return {
      key: route.key,
      path: route.path,
      redirect: route.redirect,
      action: route.action,
      match: createMatch(route.match, params),
      getLocation: createGetLocation(route.path, pathParams),
      getParamValues: createGetParamValues(pathPattern, pathParams),
      locationMatch: createLocationMatch(pathPattern),
      params: params
    };
  });
};